#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from ldap3 import SEARCH_SCOPE_BASE_OBJECT, SEARCH_SCOPE_SINGLE_LEVEL, SEARCH_SCOPE_WHOLE_SUBTREE

# import interfaces
from pyams_security.interfaces import IAuthenticationPlugin, IDirectorySearchPlugin

# import packages
from zope.schema import TextLine, Bool, Int, Choice
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from pyams_ldap import _


SEARCH_SCOPES = {SEARCH_SCOPE_BASE_OBJECT: _("Base object"),
                 SEARCH_SCOPE_SINGLE_LEVEL: _("Single level"),
                 SEARCH_SCOPE_WHOLE_SUBTREE: _("Whole subtree")}

SEARCH_SCOPES_VOCABULARY = SimpleVocabulary([SimpleTerm(v, title=t) for v, t in SEARCH_SCOPES.items()])


class ILDAPPlugin(IAuthenticationPlugin, IDirectorySearchPlugin):
    """LDAP authentication plug-in interface"""

    server_uri = TextLine(title=_("LDAP server URI"),
                          description=_("Full URI (including protocol) of LDAP server"),
                          default="ldap://localhost:389",
                          required=True)

    bind_dn = TextLine(title=_("Bind DN"),
                       description=_("DN used for LDAP bind; keep empty for anonymous"),
                       required=False)

    bind_password = TextLine(title=_("Bind password"),
                             description=_("Password used for LDAP bind"),
                             required=False)

    use_tls = Bool(title=_("Use TLS?"),
                   required=True,
                   default=False)

    use_pool = Bool(title=_("Use connections pool?"),
                    required=True,
                    default=True)

    pool_size = Int(title=_("Pool size"),
                    required=False,
                    default=10)

    pool_lifetime = Int(title=_("Pool lifetime"),
                        description=_("Duration, in seconds, of pool lifetime"),
                        required=False)

    base_dn = TextLine(title=_("Base DN"),
                       description=_("LDAP base DN"),
                       required=True)

    search_scope = Choice(title=_("Search scope"),
                          vocabulary=SEARCH_SCOPES_VOCABULARY,
                          default=SEARCH_SCOPE_WHOLE_SUBTREE,
                          required=True)

    login_attribute = TextLine(title=_("Login attribute"),
                               description=_("LDAP attribute used as user login"),
                               required=True,
                               default='uid')

    login_query = TextLine(title=_("Login query"),
                           description=_("Query template used to authenticate user "
                                         "(based on login attribute called 'login')"),
                           required=True,
                           default='(uid={login})')

    uid_attribute = TextLine(title=_("UID attribute"),
                             description=_("LDAP attribute used as principal identifier"),
                             required=True,
                             default='dn')

    uid_query = TextLine(title=_("UID query"),
                         description=_("Query template used to get principal information "
                                       "(based on UID attribute called 'login')"),
                         required=True,
                         default="(objectClass=*)")

    title_format = TextLine(title=_("Title format"),
                            description=_("Principal's title format string"),
                            required=True,
                            default='{givenName[0]} {sn[0]}')

    groups_base_dn = TextLine(title=_("Groups base DN"),
                              description=_("Base DN used to search LDAP groups; keep empty to "
                                            "disable groups usage"),
                              required=False)

    groups_search_scope = Choice(title=_("Groups search scope"),
                                 vocabulary=SEARCH_SCOPES_VOCABULARY,
                                 default=SEARCH_SCOPE_WHOLE_SUBTREE,
                                 required=False)

    groups_query = TextLine(title=_("Groups query"),
                            description=_("Query template used to get principal groups "
                                          "(based on DN and UID attributes called 'dn' "
                                          "and 'login')"),
                            required=False,
                            default="(&(objectClass=groupOfUniqueNames)(uniqueMember={dn}))")

    group_prefix = TextLine(title=_("Group prefix"),
                            description=_("Prefix used to identify groups"),
                            required=False,
                            default='group')

    group_uid_attribute = TextLine(title=_("Group UID attribute"),
                                   description=_("LDAP attribute used as group identifier"),
                                   required=False,
                                   default='dn')

    group_title_format = TextLine(title=_("Group title format"),
                                  description=_("Principal's title format string"),
                                  required=True,
                                  default='{cn[0]}')

    users_select_query = TextLine(title=_("Users select query"),
                                  description=_("Query template used to select users"),
                                  required=True,
                                  default='(|(givenName={query}*)(sn={query}*))')

    users_search_query = TextLine(title=_("Users search query"),
                                  description=_("Query template used to search users"),
                                  required=True,
                                  default='(|(givenName={query}*)(sn={query}*))')

    groups_select_query = TextLine(title=_("Groups select query"),
                                   description=_("Query template used to select groups"),
                                   required=True,
                                   default='(cn=*{query}*)')

    groups_search_query = TextLine(title=_("Groups search query"),
                                   description=_("Query template used to search groups"),
                                   required=True,
                                   default='(cn=*{query}*)')

    def get_connection(self):
        """Get LDAP connection"""
